@@ -4,10 +4,17 @@ namespace Ramsey\Uuid\Test;
 
 use Moontoast\Math\BigNumber;
 use Ramsey\Uuid\Builder\DegradedUuidBuilder;
+use Ramsey\Uuid\Codec\CodecInterface;
 use Ramsey\Uuid\Codec\TimestampFirstCombCodec;
 use Ramsey\Uuid\Converter\Number\DegradedNumberConverter;
+use Ramsey\Uuid\Converter\Time\DegradedTimeConverter;
+use Ramsey\Uuid\Converter\TimeConverterInterface;
+use Ramsey\Uuid\DegradedUuid;
 use Ramsey\Uuid\Generator\CombGenerator;
 use Ramsey\Uuid\Generator\DefaultTimeGenerator;
+use Ramsey\Uuid\Math\BrickMathCalculator;
+use Ramsey\Uuid\Type\Hexadecimal;
+use Ramsey\Uuid\Type\Time;
 use Ramsey\Uuid\Uuid;
 use Ramsey\Uuid\UuidFactory;
 use stdClass;
@@ -21,6 +28,8 @@ use stdClass;
  * ensure that the base-level functionality that satisfies 80% of use-cases
  * does not change. The remaining 20% of use-cases should refer to the README
  * for details on the easiest path to transition from 3.x to 4.x.
+ *
+ * @codingStandardsIgnoreFile
  */
 class ExpectedBehaviorTest extends TestCase
 {
@@ -32,38 +41,38 @@ class ExpectedBehaviorTest extends TestCase
         $uuid = call_user_func_array(['Ramsey\Uuid\Uuid', $method], $args);
 
         $this->assertInstanceOf('Ramsey\Uuid\UuidInterface', $uuid);
-        $this->assertInternalType('int', $uuid->compareTo(Uuid::uuid1()));
+        $this->assertIsInt($uuid->compareTo(Uuid::uuid1()));
         $this->assertNotSame(0, $uuid->compareTo(Uuid::uuid4()));
         $this->assertSame(0, $uuid->compareTo(clone $uuid));
         $this->assertFalse($uuid->equals(new stdClass()));
         $this->assertTrue($uuid->equals(clone $uuid));
-        $this->assertInternalType('string', $uuid->getBytes());
+        $this->assertIsString($uuid->getBytes());
         $this->assertInstanceOf('Ramsey\Uuid\Converter\NumberConverterInterface', $uuid->getNumberConverter());
-        $this->assertInternalType('string', $uuid->getHex());
-        $this->assertInternalType('array', $uuid->getFieldsHex());
+        $this->assertIsString((string) $uuid->getHex());
+        $this->assertIsArray($uuid->getFieldsHex());
         $this->assertArrayHasKey('time_low', $uuid->getFieldsHex());
         $this->assertArrayHasKey('time_mid', $uuid->getFieldsHex());
         $this->assertArrayHasKey('time_hi_and_version', $uuid->getFieldsHex());
         $this->assertArrayHasKey('clock_seq_hi_and_reserved', $uuid->getFieldsHex());
         $this->assertArrayHasKey('clock_seq_low', $uuid->getFieldsHex());
         $this->assertArrayHasKey('node', $uuid->getFieldsHex());
-        $this->assertInternalType('string', $uuid->getTimeLowHex());
-        $this->assertInternalType('string', $uuid->getTimeMidHex());
-        $this->assertInternalType('string', $uuid->getTimeHiAndVersionHex());
-        $this->assertInternalType('string', $uuid->getClockSeqHiAndReservedHex());
-        $this->assertInternalType('string', $uuid->getClockSeqLowHex());
-        $this->assertInternalType('string', $uuid->getNodeHex());
+        $this->assertIsString($uuid->getTimeLowHex());
+        $this->assertIsString($uuid->getTimeMidHex());
+        $this->assertIsString($uuid->getTimeHiAndVersionHex());
+        $this->assertIsString($uuid->getClockSeqHiAndReservedHex());
+        $this->assertIsString($uuid->getClockSeqLowHex());
+        $this->assertIsString($uuid->getNodeHex());
         $this->assertSame($uuid->getFieldsHex()['time_low'], $uuid->getTimeLowHex());
         $this->assertSame($uuid->getFieldsHex()['time_mid'], $uuid->getTimeMidHex());
         $this->assertSame($uuid->getFieldsHex()['time_hi_and_version'], $uuid->getTimeHiAndVersionHex());
         $this->assertSame($uuid->getFieldsHex()['clock_seq_hi_and_reserved'], $uuid->getClockSeqHiAndReservedHex());
         $this->assertSame($uuid->getFieldsHex()['clock_seq_low'], $uuid->getClockSeqLowHex());
         $this->assertSame($uuid->getFieldsHex()['node'], $uuid->getNodeHex());
-        $this->assertSame(substr($uuid->getHex(), 16), $uuid->getLeastSignificantBitsHex());
-        $this->assertSame(substr($uuid->getHex(), 0, 16), $uuid->getMostSignificantBitsHex());
+        $this->assertSame(substr((string) $uuid->getHex(), 16), $uuid->getLeastSignificantBitsHex());
+        $this->assertSame(substr((string) $uuid->getHex(), 0, 16), $uuid->getMostSignificantBitsHex());
 
         $this->assertSame(
-            $uuid->getHex(),
+            (string) $uuid->getHex(),
             $uuid->getTimeLowHex()
             . $uuid->getTimeMidHex()
             . $uuid->getTimeHiAndVersionHex()
@@ -73,7 +82,7 @@ class ExpectedBehaviorTest extends TestCase
         );
 
         $this->assertSame(
-            $uuid->getHex(),
+            (string) $uuid->getHex(),
             $uuid->getFieldsHex()['time_low']
             . $uuid->getFieldsHex()['time_mid']
             . $uuid->getFieldsHex()['time_hi_and_version']
@@ -82,11 +91,11 @@ class ExpectedBehaviorTest extends TestCase
             . $uuid->getFieldsHex()['node']
         );
 
-        $this->assertInternalType('string', $uuid->getUrn());
+        $this->assertIsString($uuid->getUrn());
         $this->assertStringStartsWith('urn:uuid:', $uuid->getUrn());
-        $this->assertSame('urn:uuid:' . $uuid->getHex(), str_replace('-', '', $uuid->getUrn()));
-        $this->assertSame($uuid->getHex(), str_replace('-', '', $uuid->toString()));
-        $this->assertSame($uuid->getHex(), str_replace('-', '', (string) $uuid));
+        $this->assertSame('urn:uuid:' . (string) $uuid->getHex(), str_replace('-', '', $uuid->getUrn()));
+        $this->assertSame((string) $uuid->getHex(), str_replace('-', '', $uuid->toString()));
+        $this->assertSame((string) $uuid->getHex(), str_replace('-', '', (string) $uuid));
 
         $this->assertSame(
             $uuid->toString(),
@@ -134,7 +143,7 @@ class ExpectedBehaviorTest extends TestCase
     {
         $uuid = Uuid::uuid1('00000fffffff', 0xffff);
 
-        $this->assertInstanceOf('DateTime', $uuid->getDateTime());
+        $this->assertInstanceOf('DateTimeInterface', $uuid->getDateTime());
         $this->assertSame('00000fffffff', $uuid->getNodeHex());
         $this->assertSame('3fff', $uuid->getClockSequenceHex());
         $this->assertSame('16383', (string) $uuid->getClockSequence());
@@ -142,11 +151,9 @@ class ExpectedBehaviorTest extends TestCase
 
     public function testUuidVersion1MethodBehavior64Bit()
     {
-        $this->skip64BitTest();
+        $uuid = Uuid::uuid1('ffffffffffff', 0xffff);
 
-        $uuid = Uuid::uuid1(0xffffffffffff, 0xffff);
-
-        $this->assertInstanceOf('DateTime', $uuid->getDateTime());
+        $this->assertInstanceOf('DateTimeInterface', $uuid->getDateTime());
         $this->assertSame('ffffffffffff', $uuid->getNodeHex());
         $this->assertSame('281474976710655', (string) $uuid->getNode());
         $this->assertSame('3fff', $uuid->getClockSequenceHex());
@@ -237,8 +244,6 @@ class ExpectedBehaviorTest extends TestCase
      */
     public function testNumericReturnValues($string)
     {
-        $this->skip64BitTest();
-
         $leastSignificantBitsHex = substr(str_replace('-', '', $string), 16);
         $mostSignificantBitsHex = substr(str_replace('-', '', $string), 0, 16);
         $leastSignificantBits = BigNumber::convertToBase10($leastSignificantBitsHex, 16);
@@ -521,18 +526,14 @@ class ExpectedBehaviorTest extends TestCase
     public function testUsingDefaultTimeGeneratorWithCustomProviders()
     {
         $nodeProvider = \Mockery::mock('Ramsey\Uuid\Provider\NodeProviderInterface', [
-            'getNode' => '0123456789ab',
+            'getNode' => new Hexadecimal('0123456789ab'),
         ]);
 
         $timeConverter = \Mockery::mock('Ramsey\Uuid\Converter\TimeConverterInterface');
         $timeConverter
             ->shouldReceive('calculateTime')
-            ->andReturnUsing(function ($seconds, $microSeconds) {
-                return [
-                    'low' => dechex($seconds),
-                    'mid' => dechex($microSeconds),
-                    'hi' => 'abcd',
-                ];
+            ->andReturnUsing(function ($seconds, $microseconds) {
+                return new Hexadecimal('abcd' . dechex($microseconds) . dechex($seconds));
             });
 
         $timeProvider = \Mockery::mock('Ramsey\Uuid\Provider\TimeProviderInterface', [
@@ -540,6 +541,7 @@ class ExpectedBehaviorTest extends TestCase
                 'sec' => 1578522046,
                 'usec' => 10000,
             ],
+            'getTime' => new Time(1578522046, 10000),
         ]);
 
         $generator = new DefaultTimeGenerator($nodeProvider, $timeConverter, $timeProvider);
@@ -620,6 +622,35 @@ class ExpectedBehaviorTest extends TestCase
         $this->assertSame(2, $uuid->getVariant());
         $this->assertSame(4, $uuid->getVersion());
         $this->assertSame($expectedBytes, $uuid->getBytes());
-        $this->assertSame($expectedHex, $uuid->getHex());
+        $this->assertSame($expectedHex, (string) $uuid->getHex());
+    }
+
+    /**
+     * @dataProvider provideUuidConstantTests
+     */
+    public function testUuidConstants($constantName, $expected)
+    {
+        $this->assertSame($expected, constant("Ramsey\\Uuid\\Uuid::{$constantName}"));
+    }
+
+    public function provideUuidConstantTests()
+    {
+        return [
+            ['NAMESPACE_DNS', '6ba7b810-9dad-11d1-80b4-00c04fd430c8'],
+            ['NAMESPACE_URL', '6ba7b811-9dad-11d1-80b4-00c04fd430c8'],
+            ['NAMESPACE_OID', '6ba7b812-9dad-11d1-80b4-00c04fd430c8'],
+            ['NAMESPACE_X500', '6ba7b814-9dad-11d1-80b4-00c04fd430c8'],
+            ['NIL', '00000000-0000-0000-0000-000000000000'],
+            ['RESERVED_NCS', 0],
+            ['RFC_4122', 2],
+            ['RESERVED_MICROSOFT', 6],
+            ['RESERVED_FUTURE', 7],
+            ['VALID_PATTERN', '^[0-9A-Fa-f]{8}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{4}-[0-9A-Fa-f]{12}$'],
+            ['UUID_TYPE_TIME', 1],
+            ['UUID_TYPE_IDENTIFIER', 2],
+            ['UUID_TYPE_HASH_MD5', 3],
+            ['UUID_TYPE_RANDOM', 4],
+            ['UUID_TYPE_HASH_SHA1', 5],
+        ];
     }
 }
